import 'dart:developer' as developer;
import 'dart:io' show Platform;

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

import 'apptics_flutter_platform_interface.dart';
import 'apptics_flutter_util.dart';
import 'appupdate/apptics_app_update_alerts.dart';
import 'appupdate/appupdate_util.dart';

/// An implementation of [AppticsFlutterPlatform] that uses method channels.

class MethodChannelAppticsFlutter extends AppticsFlutterPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('apptics_flutter');

  @override
  Future<void> setCrashCustomProperty(Map<String, dynamic> properties) async {
    if (Platform.isIOS) {
      methodChannel
          .invokeMethod('setCrashCustomProperty', {"properties": properties});
    } else if (Platform.isAndroid) {
      methodChannel
          .invokeMethod('setCrashCustomProperty', {"properties": properties});
    }
  }

  @override
  Future<void> presentPrivacyReviewPopup() async {
    if (Platform.isIOS) {
      methodChannel.invokeMethod('presentPrivacyReviewPopup');
    } else if (Platform.isAndroid) {
      methodChannel.invokeMethod('presentPrivacyReviewPopup');
    }
  }

  @override
  Future<void> openPrivacySettings() async {
    if (Platform.isIOS) {
      methodChannel.invokeMethod<void>('openPrivacySettings');
    } else if (Platform.isAndroid) {
      methodChannel.invokeMethod<void>('openPrivacySettings');
    }
  }

  @override
  Future<void> setUser(String? user_id) async {
    if (Platform.isIOS) {
      methodChannel.invokeMethod<void>('setUser', {"user_id": user_id});
    } else if (Platform.isAndroid) {
      methodChannel.invokeMethod<void>('setUser', {"userId": user_id});
    }
  }

  @override
  Future<TrackingState?> getTrackingState() async {
    int status =
        await methodChannel.invokeMethod<int>('getTrackingState') ?? -1;
    return TrackingState.getByValue(status);
  }

  @override
  Future<void> setTrackingState(TrackingState status) async {
    if (Platform.isIOS) {
      methodChannel
          .invokeMethod<void>('setTrackingState', {"status": status.value});
    } else if (Platform.isAndroid) {
      methodChannel
          .invokeMethod<void>('setTrackingState', {"status": status.value});
    }
  }

  @override
  Future<bool?> isUserLoggedIn() async {
    if (Platform.isIOS) {
      return await methodChannel.invokeMethod<bool>('isUserLoggedIn');
    } else if (Platform.isAndroid) {
      return await methodChannel.invokeMethod("isUserLoggedIn");
    }
    return null;
  }

  @override
  Future<void> addEvent(
      String? event, String? group, Map<String, dynamic>? properties) async {
    if (Platform.isIOS) {
      methodChannel.invokeMethod<void>('addEvent',
          {"event": event, "group": group, "properties": properties});
    } else if (Platform.isAndroid) {
      methodChannel.invokeMethod<void>('addEvent',
          {"event": event, "group": group, "properties": properties});
    }
  }

  @override
  Future<void> screenAttached(String screen_name) async {
    if (Platform.isIOS) {
      methodChannel
          .invokeMethod<void>('screenAttached', {"screen_name": screen_name});
    } else if (Platform.isAndroid) {
      methodChannel
          .invokeMethod<void>('screenAttached', {"screen_name": screen_name});
    }
  }

  @override
  Future<void> screenDetached(String screen_name) async {
    if (Platform.isIOS) {
      methodChannel
          .invokeMethod<void>('screenDetached', {"screen_name": screen_name});
    } else if (Platform.isAndroid) {
      methodChannel
          .invokeMethod<void>('screenDetached', {"screen_name": screen_name});
    }
  }

  @override
  Future<void> showLastSessionCrashedPopup() async {
    if (Platform.isAndroid) {
      await methodChannel.invokeMethod<String>('showLastSessionCrashedPopup');
    }
  }

  @override
  Future<void> setDefaultLanguage(String lang) async {
    if (Platform.isIOS) {
      methodChannel.invokeMethod<void>('setDefaultLanguage', {"lang": lang});
    } else if (Platform.isAndroid) {
      developer.log('method not available', name: 'Apptics');
    }
  }

  @override
  Future<void> checkAndUpdateAlert(BuildContext context) async {
    // if (true) {
      final response = await methodChannel
          .invokeMapMethod<String, dynamic>("checkAndUpdateAlert");

      ///The value returned from the Native SDK is in the form of a Map<String, dynamic>?.
      if (response != null) {
        if (response['category'] == 1) {
          /*
          mapOf<String, Any>(
            "updateid" to updateData.updateId,
            "currentversion" to updateData.currentVersion,
            "featureTitle" to updateData.featureTitle,
            "features" to updateData.features,
            "remindMeLaterText" to updateData.remindMeLaterText,
            "updateNowText" to updateData.updateNowText,
            "neverAgainText" to updateData.neverAgainText,
            "option" to updateData.option,
            "reminderDays" to updateData.reminderDays,
            "forceInDays" to updateData.forceInDays,
            "alertType" to updateData.alertType,
            "customStoreUrl" to updateData.customStoreUrl,
            "category" to 1
        )*/

          final map =
              Map<String, dynamic>.from(response.cast<String, dynamic>());
          final updateData = AppUpdateData.fromJson(map);
          Future.delayed(
              Duration.zero,
              () => AppticsInAppUpdateAlert.show(
                  context: context,
                  appUpdateData: updateData,
                  onClickUpdate: onClickUpdate,
                  onClickReminder: onClickReminder,
                  onClickIgnore: onClickIgnore));
          onSendImpressionStatus(updateData.updateId);
        } else if (response['category'] == 2) {
          /*
          mapOf<String, Any>(
                "title" to appticsAppUpdateNotSupported.title,
                "description" to appticsAppUpdateNotSupported.description,
                "continueBtTxt" to appticsAppUpdateNotSupported.continueBtTxt,
                "alertType" to appticsAppUpdateNotSupported.alertType,
                "updateId" to appticsAppUpdateNotSupported.updateId,
                "category" to 2
            )
            */

          final map =
              Map<String, dynamic>.from(response.cast<String, dynamic>());
          final updateData = AppticsAppUpdateNotSupported.fromJson(map);
          Future.delayed(
              Duration.zero,
              () => AppticsInAppUpdateAlert.showNonSupportAlert(
                  context: context,
                  appUpdateData: updateData,
                  onClick: onClickNonSupportAlert));
          onSendImpressionStatus(updateData.updateId);
        }
      }
    // }
  }

  @override
  Future<void> onClickUpdate() async {
    methodChannel.invokeMethod('onClickUpdate');
  }

  @override
  Future<void> onClickReminder() async {
    methodChannel.invokeMethod('onClickReminder');
  }

  @override
  Future<void> onClickIgnore() async {
    methodChannel.invokeMethod('onClickIgnore');
  }

  @override
  Future<void> onClickNonSupportAlert() async {
    methodChannel.invokeMethod('onClickNonSupportAlert');
  }

  @override
  Future<void> onSendImpressionStatus(String updateId) async {
    methodChannel
          .invokeMethod('onSendImpressionStatus', {'updateId': updateId});
  }

  @override
  Future<Map<String, dynamic>?> getInAppUpdateData() async {
    return await methodChannel
          .invokeMapMethod<String, dynamic>("checkAndUpdateAlert");
  }
}
